///////////////PEG/////////////////////
function Peg(gameBoard, row, col){
	this.color = -1; //set to blank
	this.enabled = false;
	this.paused = false;
	this.row = row;
	this.col = col;
	
	this.image = document.createElement("img");
	this.image.style.position = "absolute";
	this.image.style.left = 56 + col*26 + "px";
	this.image.style.top = 53 + row*20 + "px";
	this.image.setAttribute("class", "peg");
	this.image.src = "images/blank.png";

	var self = this;
	this.setColor = function(color){
		if(self.enabled && !self.paused){
			if(self.color == color || color == -1){
				self.color = -1;
				self.image.src = "images/blank.png";
			}else{
				self.color = color;
				self.image.src = colorPngs[color];
			}	
		}	
	}
	
	this.image.onclick = function(){
		self.setColor(ColorSelector.color);
	}
	
	//if we are paused or game over do not allow dragging
	this.image.ondragstart = function(event){
		if(self.paused || codeMaster.gameOver){
			return false;
		}
		dragColor = self.color;
		dragSrc = self;
		return true;
	}
		
	this.image.ondragover = function(event){
		if(self.enabled && self != dragSrc && !self.paused){
			var e = window.event;
			e.stopPropagation();
			e.preventDefault();
		}
	}
		
	this.image.ondrop = function(event){
		if(self != dragSrc){
			self.setColor(dragColor);
		}
	}
	
	this.image.onmousewheel = function(event){
		var color = ((event.wheelDelta > 0 ? -1 : 1) + self.color) % 6;
		if(color < -1){
			color = 5;
		}
		self.setColor(color);
	}
	
	gameBoard.appendChild(this.image);
}

Peg.prototype.clear = function(){
	this.image.src = "images/blank.png";
	this.color = -1;
	this.enabled = false;
	this.paused = false;
}

Peg.prototype.setEnabled = function(enable){
	this.enabled = enable;
}

Peg.prototype.pause = function(pause){
	this.paused = pause;
	if(this.paused){
		if(!this.enabled || (this.enabled && this.color != -1)){
			this.image.src = "images/disabled.png";
		}
	}else{
		this.image.src = this.color == -1 ? "images/blank.png" : colorPngs[this.color];
	}
}
///////////////PEG GROUP/////////////////////
function PegGroup(gameBoard, row){
	this.pegs = [];
	for(var i = 0; i < 4; i++){
		this.pegs.push(new Peg(gameBoard, row, i));
	}
}

PegGroup.prototype.clear = function(){
	for(var i = 0; i < 4; i++){
		this.pegs[i].clear();
	}
}

PegGroup.prototype.setEnabled = function(enable){
	for(var i = 0; i < 4; i++){
		this.pegs[i].setEnabled(enable);
	}	
}

PegGroup.prototype.getGuess = function(){
	guess = [];
	for(var i = 0; i < 4; i++){
		guess.push(this.pegs[i].color);
	}
	return guess;
}

PegGroup.prototype.pause = function(pause){
	for(var i = 0; i < 4; i++){
		this.pegs[i].pause(pause);
	}
}

PegGroup.prototype.mock = function(pegGroup){
	for(var i = 0; i < 4; i++){
		this.pegs[i].setColor(-1);
		this.pegs[i].setColor(pegGroup.pegs[i].color);
	}
}

///////////////KEY/////////////////////
function Key(gameBoard, row, subRow, subCol){
	this.image = document.createElement("img");
	this.image.style.position = "absolute";
	this.image.style.left = 20 + 8*subCol + "px";
	this.image.style.top = 53 + 7*subRow + row*20 + "px";
	this.image.src = "images/blank.png";	
	gameBoard.appendChild(this.image);
}

Key.prototype.setValue = function(red){
	if(red){
		this.image.src = "images/red-small.png";
	}else{
		this.image.src = "images/white-small.png";
	}
}

Key.prototype.clear = function(){
	this.image.src = "images/blank.png";
}


///////////////KEY GROUP/////////////////////
function KeyGroup(gameBoard, row){
	this.keys = [];
	for(var i = 0; i < 2; i++){
		for(var j = 0; j < 2; j++){
			this.keys.push(new Key(gameBoard, row, i, j));
		}
	}
}

KeyGroup.prototype.setKeys = function(reds, whites){
	var index = 0;
	for(var i = 0; i < reds; i++){
		this.keys[index++].setValue(true);
	}
	for(var i = 0; i < whites; i++){
		this.keys[index++].setValue(false);
	}
}

KeyGroup.prototype.clear = function(){
	for(var i = 0; i < 4; i++){
		this.keys[i].clear();
	}	
}


///////////////FULL ROW/////////////////////
function CodeRow(gameBoard, row, repeat){
	this.repeat = repeat;
	this.enabled = false;
	this.pegGroup = new PegGroup(gameBoard, row);
	this.keyGroup = new KeyGroup(gameBoard, row);
}

CodeRow.prototype.clear = function(){
	this.pegGroup.clear();
	this.keyGroup.clear();
	this.enabled = false;
	this.repeat.enable(false);
}

CodeRow.prototype.setEnabled = function(enable){
	this.enabled = enable;
	this.pegGroup.setEnabled(enable);
}

CodeRow.prototype.getGuess = function(){
	return this.pegGroup.getGuess();
}

CodeRow.prototype.pause = function(pause){
	this.pegGroup.pause(pause);
	if(!this.enabled){
		this.repeat.enable(!pause);
	}
}

CodeRow.prototype.mock = function(codeRow){
	this.pegGroup.mock(codeRow.pegGroup);
}

///////////////REPEAT BUTTON/////////////////////
function RepeatButton(row, action){
	this.image = document.createElement("img");
	this.image.src = "images/copy_up.png";
	this.image.style.position = "absolute";
	this.image.style.display = "none";
	this.image.style.left = "39px";
	this.image.style.top = 50 + row*20 + "px";
	document.getElementById("front").appendChild(this.image);
	var self = this;
	this.enable = function(enable){
		self.image.style.display = enable && row != 0 && showCopy ? "block" : "none";
	}
	this.image.onclick = function(){ action(row); };
}
